package jumpstart.web.components.examples.component;

import java.util.Set;

import jumpstart.business.domain.examples.Person;

import org.apache.tapestry5.BindingConstants;
import org.apache.tapestry5.ComponentAction;
import org.apache.tapestry5.ValidationTracker;
import org.apache.tapestry5.annotations.Environmental;
import org.apache.tapestry5.annotations.Parameter;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.services.FormSupport;

public class SelectPersonsValidated {

	// Parameters

	@Parameter(required = true, allowNull = false)
	@Property
	@SuppressWarnings("unused")
	private Iterable<Person> persons;

	@Parameter(required = true, allowNull = false, name = "chosen")
	@Property
	private Set<Person> chosenPersons;

	@Parameter(defaultPrefix = BindingConstants.LITERAL, value = "0")
	@Property
	private int min;

	@Parameter(defaultPrefix = BindingConstants.LITERAL, value = "false")
	@Property
	@SuppressWarnings("unused")
	private boolean disabled;

	// Screen fields

	@Property
	private Person person;

	// Generally useful bits and pieces

	@Environmental
	private FormSupport formSupport;

	@Environmental
	private ValidationTracker tracker;

	private static final ProcessSubmission PROCESS_SUBMISSION = new ProcessSubmission();

	// The code

	final void afterRender() {

		// If we are inside a form then add my action PROCESS_SUBMISSION to its list of things to do on submit.
		// It will be executed later, on submit, AFTER the components I contain are processed (including their
		// validation).

		if (formSupport != null) {
			formSupport.store(this, PROCESS_SUBMISSION);
		}
	}

	private void processSubmission() {

		// Validate. We ensured above that my sub-components will already have been processed before this.
		// Error if the number of persons chosen is less than specified by the min parameter.

		if (chosenPersons.size() < min) {
			tracker.recordError("You must choose at least " + min + " person(s).");
			return;
		}

	}

	// The Loop component will automatically call this for every row as it is rendered.
	public boolean isPersonChosen() {
		return chosenPersons.contains(person);
	}

	// The Loop component will automatically call this for every row on submit.
	public void setPersonChosen(boolean personChosen) {
		if (personChosen) {
			chosenPersons.add(person);
		}
		else {
			chosenPersons.remove(person);
		}
	}

	private static class ProcessSubmission implements ComponentAction<SelectPersonsValidated> {
		private static final long serialVersionUID = 1L;

		public void execute(SelectPersonsValidated component) {
			component.processSubmission();
		}

		@Override
		public String toString() {
			return this.getClass().getSimpleName() + ".ProcessSubmission";
		}
	};

}
