package jumpstart.web.pages.examples.ajax.componentscrud;

import jumpstart.web.components.examples.ajax.componentscrud.PersonEditor.Mode;

import org.apache.tapestry5.ajax.MultiZoneUpdate;
import org.apache.tapestry5.annotations.InjectComponent;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Zone;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.services.Request;

public class Persons {

	// Screen fields

	@Property
	@SuppressWarnings("unused")
	private Mode mode;

	@Property
	private Long personId;

	@Property
	@Persist
	private boolean highlightZoneUpdates;

	@Property
	@SuppressWarnings("unused")
	private Long highlightPersonId;

	// Generally useful bits and pieces

	@InjectComponent
	private Zone listZone;

	@InjectComponent
	private Zone editorZone;

	@Inject
	private Request request;

	// The code

	// setupRender() is called by Tapestry right before it starts rendering the page.

	void setupRender() {
		highlightPersonId = personId;
	}

	// /////////////////////////////////////////////////////////////////////
	// FILTER
	// /////////////////////////////////////////////////////////////////////

	// Handle event "filter" from component "list"

	Object onFilterFromList(Long highlightPersonId) {
		this.highlightPersonId = highlightPersonId;
		return request.isXHR() ? listZone.getBody() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// CREATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toCreate" from this page

	Object onToCreate() {
		mode = Mode.CREATE;
		personId = null;
		highlightPersonId = null;
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	// Handle event "cancelCreate" from component "editor"

	Object onCancelCreateFromEditor() {
		mode = null;
		personId = null;
		highlightPersonId = null;
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// Handle event "successfulCreate" from component "editor"

	Object onSuccessfulCreateFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		highlightPersonId = personId;
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	// Handle event "failedCreate" from component "editor"

	Object onFailedCreateFromEditor() {
		mode = Mode.CREATE;
		personId = null;
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// REVIEW
	// /////////////////////////////////////////////////////////////////////

	// Handle event "selected" from component "list"

	Object onSelectedFromList(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		highlightPersonId = personId;
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// UPDATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toUpdate" from component "editor"

	Object onToUpdateFromEditor(Long personId) {
		mode = Mode.UPDATE;
		this.personId = personId;
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// Handle event "cancelUpdate" from component "editor"

	Object onCancelUpdateFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// Handle event "successfulUpdate" from component "editor"

	Object onSuccessfulUpdateFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		highlightPersonId = personId;
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	// Handle event "failedUpdate" from component "editor"

	Object onFailedUpdateFromEditor(Long personId) {
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// DELETE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "successfulDelete" from component "editor"

	Object onSuccessfulDeleteFromEditor(Long personId) {
		mode = null;
		this.personId = null;
		highlightPersonId = null;
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	// Handle event "failedDelete" from component "editor"

	Object onFailedDeleteFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		highlightPersonId = personId;
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// GETTERS
	// /////////////////////////////////////////////////////////////////////

	private MultiZoneUpdate getListAndEditorZones() {
		return new MultiZoneUpdate("listZone", listZone.getBody()).add("editorZone", editorZone.getBody());
	}

	public String getZoneUpdateFunction() {
		return highlightZoneUpdates ? "highlight" : "show";
	}

}
