package jumpstart.web.pages.examples.ajax.gracefulcomponentscrud;

import jumpstart.web.components.examples.ajax.gracefulcomponentscrud.PersonEditor.Mode;

import org.apache.tapestry5.EventContext;
import org.apache.tapestry5.ajax.MultiZoneUpdate;
import org.apache.tapestry5.annotations.InjectComponent;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Zone;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.services.Request;

public class Persons {

	// The activation context

	@Property
	private Mode mode;

	@Property
	private Long personId;

	// Screen fields

	@Property
	@Persist
	private boolean highlightZoneUpdates;

	@Property
	@SuppressWarnings("unused")
	private Long highlightPersonId;

	// Generally useful bits and pieces

	@InjectComponent
	private Zone listZone;

	@InjectComponent
	private Zone editorZone;

	@Inject
	private Request request;

	// The code

	// onPassivate() is called by Tapestry to get the activation context to put in the URL.

	Object[] onPassivate() {

		if (mode == null) {
			return null;
		}
		else if (mode == Mode.CREATE) {
			return new Object[] { mode };
		}
		else if (mode == Mode.REVIEW || mode == Mode.UPDATE || mode == Mode.CONFIRM_DELETE) {
			return new Object[] { mode, personId };
		}
		else {
			throw new IllegalStateException(mode.toString());
		}

	}

	// onActivate() is called by Tapestry to pass in the activation context from the URL.

	void onActivate(EventContext ec) {

		if (ec.getCount() == 0) {
			mode = null;
			personId = null;
		}
		else if (ec.getCount() == 1) {
			mode = ec.get(Mode.class, 0);
			personId = null;
		}
		else {
			mode = ec.get(Mode.class, 0);
			personId = ec.get(Long.class, 1);
		}

	}

	// setupRender() is called by Tapestry right before it starts rendering the page.

	void setupRender() {
		highlightPersonId = personId;
	}

	// /////////////////////////////////////////////////////////////////////
	// FILTER
	// /////////////////////////////////////////////////////////////////////

	// Handle event "filter" from component "list"

	Object onFilterFromList(Long highlightPersonId) {
		this.highlightPersonId = highlightPersonId;
		return request.isXHR() ? listZone.getBody() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// CREATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toCreate" from this page

	Object onToCreate() {
		mode = Mode.CREATE;
		personId = null;
		highlightPersonId = null;
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	// Handle event "cancelCreate" from component "editor"

	Object onCancelCreateFromEditor() {
		mode = null;
		personId = null;
		highlightPersonId = null;
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// Handle event "successfulCreate" from component "editor"

	Object onSuccessfulCreateFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		highlightPersonId = personId;
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	// Handle event "failedCreate" from component "editor"

	Object onFailedCreateFromEditor() {
		mode = Mode.CREATE;
		personId = null;
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// REVIEW
	// /////////////////////////////////////////////////////////////////////

	// Handle event "selected" from component "list"

	Object onSelectedFromList(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		highlightPersonId = personId;
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// UPDATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toUpdate" from component "editor"

	Object onToUpdateFromEditor(Long personId) {
		mode = Mode.UPDATE;
		this.personId = personId;
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// Handle event "cancelUpdate" from component "editor"

	Object onCancelUpdateFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// Handle event "successfulUpdate" from component "editor"

	Object onSuccessfulUpdateFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		highlightPersonId = personId;
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	// Handle event "failedUpdate" from component "editor"

	Object onFailedUpdateFromEditor(Long personId) {
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// DELETE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "successfulDelete" from component "editor"

	Object onSuccessfulDeleteFromEditor(Long personId) {
		mode = null;
		this.personId = null;
		highlightPersonId = null;
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	// Handle event "failedDelete" from component "editor"

	Object onFailedDeleteFromEditor(Long personId) {
		this.personId = personId;
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// CONFIRM DELETE - used only when JavaScript is disabled.
	// /////////////////////////////////////////////////////////////////////

	// Handle event "cancelConfirmDelete" from component "editor"

	Object onCancelConfirmDeleteFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		highlightPersonId = personId;
		return null;
	}

	// /////////////////////////////////////////////////////////////////////
	// GETTERS
	// /////////////////////////////////////////////////////////////////////

	private MultiZoneUpdate getListAndEditorZones() {
		return new MultiZoneUpdate("listZone", listZone.getBody()).add("editorZone", editorZone.getBody());
	}

	public String getZoneUpdateFunction() {
		return highlightZoneUpdates ? "highlight" : "show";
	}

}
