// Based on http://wiki.apache.org/tapestry/JEE-Annotation, adapted to Tapestry 5.2 (from 5.3) and EJB 3.0 (from 3.1).

package jumpstart.web.services;

import javax.ejb.EJB;

import jumpstart.client.BusinessServicesLocator;
import jumpstart.util.StringUtil;

import org.apache.tapestry5.model.MutableComponentModel;
import org.apache.tapestry5.services.ClassTransformation;
import org.apache.tapestry5.services.ComponentClassTransformWorker;
import org.apache.tapestry5.services.TransformField;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Inject an EJB into tapestry sources
 */
public class EJBAnnotationWorker implements ComponentClassTransformWorker {
	private static final Logger logger = LoggerFactory.getLogger(EJBAnnotationWorker.class);

	private BusinessServicesLocator locator;

	public EJBAnnotationWorker() {
		locator = new BusinessServicesLocator(logger);
	}

	@Override
	public void transform(ClassTransformation classTransformation, MutableComponentModel model) {

		for (TransformField field : classTransformation.matchFieldsWithAnnotation(EJB.class)) {
			final EJB annotation = field.getAnnotation(EJB.class);

			if (StringUtil.isNotEmpty(annotation.name())) {
				throw new RuntimeException(
						"This implementation of the @EJB annotation does not support the name parameter. Found name = \""
								+ annotation.name() + "\".");
			}
			else if (StringUtil.isNotEmpty(annotation.beanName())) {
				throw new RuntimeException(
						"This implementation of the @EJB annotation does not support the beanName parameter. Found beanName = \""
								+ annotation.beanName() + "\".");
			}
			else if (StringUtil.isNotEmpty(annotation.mappedName())) {
				throw new RuntimeException(
						"This implementation of the @EJB annotation does not support the mappedName parameter. Found mappedName = \""
								+ annotation.mappedName() + "\".");
			}

			String fieldType = field.getType();
			Object injectionValue = locator.getService(fieldType);

			if (injectionValue != null) {
				field.inject(injectionValue);
				field.claim(annotation);
			}
		}
	}

}
